package sun.management.snmp;

import java.io.IOException;
import sun.management.FileSystem;
import com.sun.jmx.snmp.daemon.CommunicationException;
import sun.management.snmp.util.JvmContextFactory;
import com.sun.jmx.snmp.IPAcl.SnmpAcl;
import java.net.UnknownHostException;
import sun.management.AgentConfigurationError;
import java.util.Properties;
import java.util.Enumeration;
import sun.management.snmp.jvminstr.NotificationTargetImpl;
import java.net.InetAddress;
import sun.management.Agent;
import java.util.ArrayList;
import sun.management.snmp.jvminstr.NotificationTarget;
import java.util.List;
import com.sun.jmx.snmp.InetAddressAcl;
import java.io.File;
import sun.management.snmp.jvminstr.JVM_MANAGEMENT_MIB_IMPL;
import com.sun.jmx.snmp.daemon.SnmpAdaptorServer;
import sun.management.snmp.util.MibLogger;

public final class AdaptorBootstrap
{
    private static final MibLogger log;
    private SnmpAdaptorServer adaptor;
    private JVM_MANAGEMENT_MIB_IMPL jvmmib;
    
    private AdaptorBootstrap(SnmpAdaptorServer adaptor, JVM_MANAGEMENT_MIB_IMPL jvmmib) {
        this.jvmmib = jvmmib;
        this.adaptor = adaptor;
    }
    
    private static String getDefaultFileName(String str) {
        String separator = File.separator;
        return System.getProperty("java.home") + separator + "lib" + separator + "management" + separator + str;
    }
    
    private static List<NotificationTarget> getTargetList(InetAddressAcl inetAddressAcl, int n) {
        ArrayList<Object> list = new ArrayList();
        if (inetAddressAcl != null) {
            if (AdaptorBootstrap.log.isDebugOn()) {
                AdaptorBootstrap.log.debug("getTargetList", Agent.getText("jmxremote.AdaptorBootstrap.getTargetList.processing"));
            }
            Enumeration<Object> trapDestinations = inetAddressAcl.getTrapDestinations();
            while (trapDestinations.hasMoreElements()) {
                InetAddress inetAddress = (InetAddress)trapDestinations.nextElement();
                Enumeration<Object> trapCommunities = inetAddressAcl.getTrapCommunities(inetAddress);
                while (trapCommunities.hasMoreElements()) {
                    NotificationTargetImpl e = new NotificationTargetImpl(inetAddress, n, (String)trapCommunities.nextElement());
                    if (AdaptorBootstrap.log.isDebugOn()) {
                        AdaptorBootstrap.log.debug("getTargetList", Agent.getText("jmxremote.AdaptorBootstrap.getTargetList.adding", new String[] { e.toString() }));
                    }
                    list.add(e);
                }
            }
            return list;
        }
        else {
            return list;
        }
    }
    
    public static synchronized AdaptorBootstrap initialize() {
        Properties loadManagementProperties = Agent.loadManagementProperties();
        if (loadManagementProperties == null) {
            return null;
        }
        return initialize(loadManagementProperties.getProperty("com.sun.management.snmp.port"), loadManagementProperties);
    }
    
    public static synchronized AdaptorBootstrap initialize(String s, Properties properties) {
        if (s.length() == 0) {
            s = "161";
        }
        int int1;
        try {
            int1 = Integer.parseInt(s);
        }
        catch (NumberFormatException ex) {
            throw new AgentConfigurationError("agent.err.invalid.snmp.port", ex, new String[] { s });
        }
        if (int1 < 0) {
            throw new AgentConfigurationError("agent.err.invalid.snmp.port", new String[] { s });
        }
        String property = properties.getProperty("com.sun.management.snmp.trap", "162");
        int int2;
        try {
            int2 = Integer.parseInt(property);
        }
        catch (NumberFormatException ex2) {
            throw new AgentConfigurationError("agent.err.invalid.snmp.trap.port", ex2, new String[] { property });
        }
        if (int2 < 0) {
            throw new AgentConfigurationError("agent.err.invalid.snmp.trap.port", new String[] { property });
        }
        String property2 = properties.getProperty("com.sun.management.snmp.interface", "localhost");
        String property3 = properties.getProperty("com.sun.management.snmp.acl.file", getDefaultFileName("snmp.acl"));
        int booleanValue = (boolean)Boolean.valueOf(properties.getProperty("com.sun.management.snmp.acl", "true"));
        if (!(booleanValue == 0)) {
            checkAclFile(property3);
        }
        AdaptorBootstrap adaptorBootstrap;
        try {
            adaptorBootstrap = getAdaptorBootstrap(int1, int2, property2, (boolean)booleanValue, property3);
        }
        catch (Exception ex3) {
            throw new AgentConfigurationError("agent.err.exception", ex3, new String[] { ex3.getMessage() });
        }
        return adaptorBootstrap;
    }
    
    private static AdaptorBootstrap getAdaptorBootstrap(int i, int n, String host, boolean b, String str) {
        InetAddress byName;
        try {
            byName = InetAddress.getByName(host);
        }
        catch (UnknownHostException ex) {
            throw new AgentConfigurationError("agent.err.unknown.snmp.interface", ex, new String[] { host });
        }
        if (AdaptorBootstrap.log.isDebugOn()) {
            AdaptorBootstrap.log.debug("initialize", Agent.getText("jmxremote.AdaptorBootstrap.getTargetList.starting\n\tcom.sun.management.snmp.port=" + i + "\n\t" + "com.sun.management.snmp.trap" + "=" + n + "\n\t" + "com.sun.management.snmp.interface" + "=" + byName + (b ? ("\n\tcom.sun.management.snmp.acl.file=" + str) : "\n\tNo ACL") + ""));
        }
        SnmpAcl snmpAcl;
        try {
            snmpAcl = (b ? new SnmpAcl(System.getProperty("user.name"), str) : null);
        }
        catch (UnknownHostException ex2) {
            throw new AgentConfigurationError("agent.err.unknown.snmp.interface", ex2, new String[] { ex2.getMessage() });
        }
        SnmpAdaptorServer snmpAdaptor = new SnmpAdaptorServer(snmpAcl, i, byName);
        snmpAdaptor.setUserDataFactory(new JvmContextFactory());
        snmpAdaptor.setTrapPort(n);
        JVM_MANAGEMENT_MIB_IMPL o = new JVM_MANAGEMENT_MIB_IMPL();
        try {
            o.init();
        }
        catch (IllegalAccessException ex3) {
            throw new AgentConfigurationError("agent.err.snmp.mib.init.failed", ex3, new String[] { ex3.getMessage() });
        }
        o.addTargets(getTargetList(snmpAcl, n));
        try {
            snmpAdaptor.start(Long.MAX_VALUE);
        }
        catch (Exception ex4) {
            if ((ex4 instanceof CommunicationException)) {
                Throwable cause = ex4.getCause();
                if (cause != null) {
                    ex4 = cause;
                }
            }
            throw new AgentConfigurationError("agent.err.snmp.adaptor.start.failed", ex4, new String[] { byName + ":" + i, "(" + ex4.getMessage() + ")" });
        }
        if (!snmpAdaptor.isActive()) {
            throw new AgentConfigurationError("agent.err.snmp.adaptor.start.failed", new String[] { byName + ":" + i });
        }
        try {
            snmpAdaptor.addMib(o);
            o.setSnmpAdaptor(snmpAdaptor);
        }
        catch (RuntimeException ex5) {
            new AdaptorBootstrap(snmpAdaptor, o).terminate();
            throw ex5;
        }
        AdaptorBootstrap.log.debug("initialize", Agent.getText("jmxremote.AdaptorBootstrap.getTargetList.initialize1"));
        AdaptorBootstrap.log.config("initialize", Agent.getText("jmxremote.AdaptorBootstrap.getTargetList.initialize2", new String[] { byName.toString(), Integer.toString(snmpAdaptor.getPort()) }));
        return new AdaptorBootstrap(snmpAdaptor, o);
    }
    
    private static void checkAclFile(String pathname) {
        if (pathname == null || pathname.length() == 0) {
            throw new AgentConfigurationError("agent.err.acl.file.notset");
        }
        File file = new File(pathname);
        if (!file.exists()) {
            throw new AgentConfigurationError("agent.err.acl.file.notfound", new String[] { pathname });
        }
        if (!file.canRead()) {
            throw new AgentConfigurationError("agent.err.acl.file.not.readable", new String[] { pathname });
        }
        FileSystem open = FileSystem.open();
        try {
            if (open.supportsFileSecurity(file) && !open.isAccessUserOnly(file)) {
                throw new AgentConfigurationError("agent.err.acl.file.access.notrestricted", new String[] { pathname });
            }
        }
        catch (IOException ex) {
            throw new AgentConfigurationError("agent.err.acl.file.read.failed", new String[] { pathname });
        }
        return;
    }
    
    public synchronized int getPort() {
        if (this.adaptor != null) {
            return this.adaptor.getPort();
        }
        return 0;
    }
    
    public synchronized void terminate() {
        if (this.adaptor == null) {
            return;
        }
        try {
            this.jvmmib.terminate();
        }
        catch (Exception ex) {
            AdaptorBootstrap.log.debug("jmxremote.AdaptorBootstrap.getTargetList.terminate", ex.toString());
        }
        finally {
            this.jvmmib = null;
        }
        try {
            this.adaptor.stop();
        }
        finally {
            this.adaptor = null;
        }
        return;
    }
    
    static {
        log = new MibLogger(AdaptorBootstrap.class);
    }
    
    public interface DefaultValues
    {
        public static final String PORT = "161";
        public static final String CONFIG_FILE_NAME = "management.properties";
        public static final String TRAP_PORT = "162";
        public static final String USE_ACL = "true";
        public static final String ACL_FILE_NAME = "snmp.acl";
        public static final String BIND_ADDRESS = "localhost";
    }
    
    public interface PropertyNames
    {
        public static final String PORT = "com.sun.management.snmp.port";
        public static final String CONFIG_FILE_NAME = "com.sun.management.config.file";
        public static final String TRAP_PORT = "com.sun.management.snmp.trap";
        public static final String USE_ACL = "com.sun.management.snmp.acl";
        public static final String ACL_FILE_NAME = "com.sun.management.snmp.acl.file";
        public static final String BIND_ADDRESS = "com.sun.management.snmp.interface";
    }
}
